==> ns-1.0a17.tar.gz: ns-1.0a17/loss-policy.cc <==

/*
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the Network Research
 * 	Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: loss-policy.cc,v 1.2 95/12/19 19:40:23 kfall Exp $ (LBL)";
#endif

#include <string.h>
#include "link.h"
#include "random.h"

/*
 * Loss policies for different lossy linke
 */

class UniformLossyLink : public LossyLink {
protected:
	int packet_policy(Packet*);	// 1->accept, 0->drop
	double loss_prob_;
public:
	UniformLossyLink();
};

static class UniformLossyLinkMatcher : public Matcher {
public:
        UniformLossyLinkMatcher() : Matcher("link") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "lossy-uniform") == 0)
                        return (new UniformLossyLink);
                return (0);
        }
} matcher_uniform_lossy;

UniformLossyLink::UniformLossyLink()
{
	link_real("ns_lossy_uniform", "loss_prob", &loss_prob_, 0);
}

UniformLossyLink::packet_policy(Packet *p)
{
	double val = Random::uniform();

	if (val > loss_prob_) {
		return (1);
	}

	return (0);
}

/*
 * deterministic discard
 *	keeps a list of 'PacketMatchers' describing essentially a TCP connection
 *	for each connection, a list of packet #s are "hotlisted".  The nth
 *	pkt matching the appropriate connection id fields is dropped
 */

class PacketMatcher;
class DeterministicLossyLink : public LossyLink {
protected:
	command(int argv, const char*const* argv);
	int build_filter(const char *);
	int packet_policy(Packet*);	// 1->accept, 0->drop
	PacketMatcher *find_connection(Packet *);
	PacketMatcher *pmlist_;	// list of 'filters'
public:
	DeterministicLossyLink();
};

class PacketMatcher;
static class DeterministicLossyLinkMatcher : public Matcher {
public:
        DeterministicLossyLinkMatcher() : Matcher("link") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "lossy-det") == 0)
                        return (new DeterministicLossyLink);
                return (0);
        }
} matcher_det_lossy;

struct NumList {
	struct NumList *next_;
	int val_;
	inline NumList(int sn) { val_ = sn; };
};

class PacketMatcher {
        u_short src_;           /* source node (for routers) */
        u_short sport_;         /* source port */
        u_short dst_;           /* destination node (for routers) */
        u_short dport_;         /* destination port */
	NumList		*nl_;
	int	cnt_;		/* # packets so far matching this matcher */
public:
	int match(Packet *);
	void setnum(int);
	PacketMatcher(u_short src, u_short sport, u_short dst, u_short dport);
	PacketMatcher	*next_;
};

int
PacketMatcher::match(Packet *p)
{
	if ((p->src_ == src_) &&
	    (p->dst_ == dst_) &&
	    (p->sport_ == sport_) &&
	    (p->dport_ == dport_)) {

		cnt_++;

		NumList *nlp = nl_;
		while (nlp != NULL) {
			if (cnt_ == nlp->val_)
				return (1);	// matched
			nlp = nlp->next_;
		}
	}
	return (0);	// no match
}

void
PacketMatcher::setnum(int seqn)
{
	NumList *nlp = nl_;
	nl_ = new NumList(seqn);
	nl_->next_ = nlp;
}

PacketMatcher::PacketMatcher(u_short src, u_short sp, u_short dst, u_short dp)
{
	src_ = src;
	sport_ = sp;
	dst_ = dst;
	dport_ = dp;

	nl_ = NULL;
	next_ = NULL;
	cnt_ = 0;
}

DeterministicLossyLink::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "filter") == 0) {
			return (build_filter(argv[2]));
		}
	}
	return(Link::command(argc, argv));
}

DeterministicLossyLink::DeterministicLossyLink()
{
	pmlist_ = NULL;
}

/* something of the form "src.sport dst.dport seqno" */
int
DeterministicLossyLink::build_filter(const char *filt)
{
	int src, sport;
	int dst, dport;
	int seq, n;
	register char *p;

	n = sscanf(filt, "%d.%d %d.%d %d",
		&src, &sport,
		&dst, &dport, &seq);

	if (n != 5)
		return (TCL_ERROR);


	// create new packet matcher
        PacketMatcher *pm, *pm0;

        pm = new PacketMatcher(src, sport, dst, dport);
	pm->setnum(seq);
        pm0 = pmlist_;
        pmlist_ = pm;
        pm->next_ = pm0;

	return(TCL_OK);
}

PacketMatcher *
DeterministicLossyLink::find_connection(Packet *p)
{
	PacketMatcher *pm = pmlist_;

	while (pm != NULL) {
		if (pm->match(p))
			return(pm);
		pm = pm->next_;
	}

	return (NULL);
}

int
DeterministicLossyLink::packet_policy(Packet *p)
{

	if (find_connection(p) != NULL)
		return (0);	// drop it
	return (1);		// didn't find it, so don't drop it
}

==> ns-1.1.tar.gz: ns-1.1/loss-policy.cc <==

/*
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 * 	This product includes software developed by the Network Research
 * 	Group at Lawrence Berkeley National Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static char rcsid[] =
    "@(#) $Header: loss-policy.cc,v 1.3 96/01/13 16:04:59 mccanne Exp $ (LBL)";
#endif

#include <string.h>
#include "link.h"
#include "random.h"

/*
 * Loss policies for different lossy linke
 */

class UniformLossyLink : public LossyLink {
protected:
	int packet_policy(Packet*);	// 1->accept, 0->drop
	double loss_prob_;
public:
	UniformLossyLink();
};

static class UniformLossyLinkMatcher : public Matcher {
public:
        UniformLossyLinkMatcher() : Matcher("link") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "lossy-uniform") == 0)
                        return (new UniformLossyLink);
                return (0);
        }
} matcher_uniform_lossy;

UniformLossyLink::UniformLossyLink()
{
	link_real("ns_lossy_uniform", "loss_prob", &loss_prob_, 0);
}

UniformLossyLink::packet_policy(Packet *p)
{
	double val = Random::uniform();

	if (val > loss_prob_) {
		return (1);
	}

	return (0);
}

/*
 * deterministic discard
 *	keeps a list of 'PacketMatchers' describing essentially a TCP connection
 *	for each connection, a list of packet #s are "hotlisted".  The nth
 *	pkt matching the appropriate connection id fields is dropped
 */

class PacketMatcher;
class DeterministicLossyLink : public LossyLink {
protected:
	int command(int argc, const char*const* argv);
	int build_filter(const char *);
	int packet_policy(Packet*);	// 1->accept, 0->drop
	PacketMatcher *find_connection(Packet *);
	PacketMatcher *pmlist_;	// list of 'filters'
public:
	DeterministicLossyLink();
};

class PacketMatcher;
static class DeterministicLossyLinkMatcher : public Matcher {
public:
        DeterministicLossyLinkMatcher() : Matcher("link") {}
        TclObject* match(const char* id) {
                if (strcasecmp(id, "lossy-det") == 0)
                        return (new DeterministicLossyLink);
                return (0);
        }
} matcher_det_lossy;

struct NumList {
	struct NumList *next_;
	int val_;
	inline NumList(int sn) { val_ = sn; };
};

class PacketMatcher {
        u_short src_;           /* source node (for routers) */
        u_short sport_;         /* source port */
        u_short dst_;           /* destination node (for routers) */
        u_short dport_;         /* destination port */
	NumList		*nl_;
	int	cnt_;		/* # packets so far matching this matcher */
public:
	int match(Packet *);
	void setnum(int);
	PacketMatcher(u_short src, u_short sport, u_short dst, u_short dport);
	PacketMatcher	*next_;
};

int
PacketMatcher::match(Packet *p)
{
	if ((p->src_ == src_) &&
	    (p->dst_ == dst_) &&
	    (p->sport_ == sport_) &&
	    (p->dport_ == dport_)) {

		cnt_++;

		NumList *nlp = nl_;
		while (nlp != NULL) {
			if (cnt_ == nlp->val_)
				return (1);	// matched
			nlp = nlp->next_;
		}
	}
	return (0);	// no match
}

void
PacketMatcher::setnum(int seqn)
{
	NumList *nlp = nl_;
	nl_ = new NumList(seqn);
	nl_->next_ = nlp;
}

PacketMatcher::PacketMatcher(u_short src, u_short sp, u_short dst, u_short dp)
{
	src_ = src;
	sport_ = sp;
	dst_ = dst;
	dport_ = dp;

	nl_ = NULL;
	next_ = NULL;
	cnt_ = 0;
}

int DeterministicLossyLink::command(int argc, const char*const* argv)
{
	if (argc == 3) {
		if (strcmp(argv[1], "filter") == 0) {
			return (build_filter(argv[2]));
		}
	}
	return(Link::command(argc, argv));
}

DeterministicLossyLink::DeterministicLossyLink()
{
	pmlist_ = NULL;
}

/* something of the form "src.sport dst.dport seqno" */
int
DeterministicLossyLink::build_filter(const char *filt)
{
	int src, sport;
	int dst, dport;
	int seq, n;
	register char *p;

	n = sscanf(filt, "%d.%d %d.%d %d",
		&src, &sport,
		&dst, &dport, &seq);

	if (n != 5)
		return (TCL_ERROR);


	// create new packet matcher
        PacketMatcher *pm, *pm0;

        pm = new PacketMatcher(src, sport, dst, dport);
	pm->setnum(seq);
        pm0 = pmlist_;
        pmlist_ = pm;
        pm->next_ = pm0;

	return(TCL_OK);
}

PacketMatcher *
DeterministicLossyLink::find_connection(Packet *p)
{
	PacketMatcher *pm = pmlist_;

	while (pm != NULL) {
		if (pm->match(p))
			return(pm);
		pm = pm->next_;
	}

	return (NULL);
}

int
DeterministicLossyLink::packet_policy(Packet *p)
{

	if (find_connection(p) != NULL)
		return (0);	// drop it
	return (1);		// didn't find it, so don't drop it
}
